import time
import requests
import json


class KycApiHelloWorld(object):

    def __init__(self):
        self.time_of_run = time.strftime('%y%m%d%H%M%S')
        # add your own user_id here
        self.application_secret = '{secret}'
        # a new application (and user_key) can be added on the dashboard; https://developer.kompany.com

        # switch - for json responses to be printed to the console (True/False)
        self.print_details_to_console_flag = False

        # switch - for json responses to be saved as files
        self.save_json_response_to_disk_flag = True

        # switch - do indexSearch or liveSearch
        self.searchType = 'indexSearch'
        # self.searchType = 'liveSearch'

        # switch - do dataset request as "full" or as "refresh"
        # self.datasetType = "refresh"
        self.datasetType = "full"

        # set number of possible retrials and delay time for product order
        self.max_try_count = 5
        self.try_delay_seconds = 5

        self.host_endpoint = 'https://api.kompany.com/api/v2/'
        self.header_params = {'Content-Type': 'application/json', 'user_key': self.application_secret, 'charset': 'utf-8'}

    def print_to_console(self, string_to_print):
        if self.print_details_to_console_flag:
            print(string_to_print)

    def save_file(self, name_of_file, string_to_save, type_of_file):
        if type_of_file == '.json' and self.save_json_response_to_disk_flag:
            try:
                filename = name_of_file + self.time_of_run + type_of_file
                file = open(filename, "w")
                file.write(string_to_save)
                file.close()
            except IOError as e:
                print("ERROR when trying to saved file to disk")
                print(e)
        elif type_of_file == '.pdf':
            try:
                filename = name_of_file + self.time_of_run + type_of_file
                file = open(filename, "wb")
                file.write(string_to_save)
                file.close()
            except IOError as e:
                print("ERROR when trying to saved file to disk")
                print(e)
        else:
            print("ERROR - save_file - wrong parameter")
            quit()
        return True

    def index_search(self, country_code, search_method, search_value):
        if self.searchType == 'indexSearch':
            search_data = dict()
            search_data["countryCode"] = country_code
            search_data["searchMethod"] = search_method
            search_data["searchValue"] = search_value

            search_url = self.host_endpoint + 'company/indexSearch'
            search_response = requests.post(
                    url=search_url,
                    headers=self.header_params,
                    json=search_data,
                    verify=False)
            search_response = json.dumps(search_response.json(), indent=4)

            self.print_to_console(search_response)
            self.save_file('1_1_response_index_search_', search_response, '.json')
            return json.loads(search_response)
        else:
            return None

    def live_search(self, country_code, search_method, search_value):
        if self.searchType == 'liveSearch':
            search_data = dict()
            search_data["countryCode"] = country_code
            search_data["searchMethod"] = search_method
            search_data["searchValue"] = search_value

            search_url = self.host_endpoint + 'company/liveSearch'
            search_response = requests.post(
                    url=search_url,
                    headers=self.header_params,
                    json=search_data,
                    verify=False)
            search_response = json.dumps(search_response.json(), indent=4)

            self.print_to_console(search_response)
            self.save_file('1_2_response_live_search_', search_response, '.json')
            return json.loads(search_response)
        else:
            return None

    def dataset(self, kompany_id, dataset_name, dataset_options=None):
        dataset_data = dict()
        dataset_data["kompanyId"] = kompany_id
        dataset_data["datasetName"] = dataset_name
        dataset_data["datasetOptions"] = dataset_options

        dataset_url = self.host_endpoint + 'company/dataset'
        dataset_response = requests.post(
                url=dataset_url,
                headers=self.header_params,
                json=dataset_data,
                verify=False)
        dataset_response = json.dumps(dataset_response.json(), indent=4)

        self.print_to_console(dataset_response)
        self.save_file('2_1_response_dataset_', dataset_response, '.json')
        return json.loads(dataset_response)

    def catalog(self, country_code):

        catalog_url = self.host_endpoint + 'system/catalog/{countryCode}'
        catalog_url = catalog_url.replace("{countryCode}", country_code)
        catalog_response = requests.get(
                url=catalog_url,
                headers=self.header_params,
                verify=False)
        catalog_response = json.dumps(catalog_response.json(), indent=4)

        self.print_to_console(catalog_response)
        self.save_file('3_1_response_catalog_', catalog_response, '.json')
        return json.loads(catalog_response)

    def product_search(self, kompany_id, product_type):
        product_search_url = self.host_endpoint + 'product/search/{kompanyId}?productType={productType}'
        product_search_url = product_search_url.replace('{kompanyId}', kompany_id)\
            .replace('{productType}', product_type)

        product_search_response = requests.get(
                url=product_search_url,
                headers=self.header_params,
                verify=False)
        product_search_response = json.dumps(product_search_response.json(), indent=4)

        self.print_to_console(product_search_response)
        self.save_file('3_3_response_product_search_', product_search_response, '.json')
        return json.loads(product_search_response)

    def product_order(self, kompany_id, product_type, order_options=None):
        product_data = dict()
        product_data["kompanyId"] = kompany_id
        product_data["productType"] = product_type
        product_data["productOptionId"] = order_options

        product_url = self.host_endpoint + 'product/order'
        product_response = requests.post(
                url=product_url,
                headers=self.header_params,
                json=product_data,
                verify=False)
        product_response = json.dumps(product_response.json(), indent=4)

        self.print_to_console(product_response)
        self.save_file('4_1_response_product_order_', product_response, '.json')
        return json.loads(product_response)

    def product_bookshelf(self, order_id, try_counter):
        product_url_get = self.host_endpoint + 'product/order'
        product_url_get = product_url_get + '/' + order_id
        product_response_get = requests.get(
            url=product_url_get,
            headers=self.header_params,
            verify=False)
        product_response_get = json.dumps(product_response_get.json(), indent=4)

        self.print_to_console(product_response_get)
        self.save_file('4_3_response_product_order_get_' + try_counter + '_', product_response_get, '.json')
        return json.loads(product_response_get)

    def product_content_retrieval(self, order_id):
        product_content_url = self.host_endpoint + 'product/order/{orderId}/content'
        product_content_url = product_content_url.replace("{orderId}", order_id)
        self.header_params['accept'] = '*/*'
        self.header_params['mimeType'] = 'application/pdf'

        product_content_response_get = requests.get(
            url=product_content_url,
            headers=self.header_params,
            verify=False)

        self.save_file('4_4_response_product_order_get_content_', product_content_response_get.content, '.pdf')
        return True

    def business_logic(self):

        if self.application_secret == "{secret}":
            print("Termination because api key has not been set (application_secret)")
            quit()

        # ------------------------------------------------------------------------------
        print("1. identifying the company")
        # ------------------------------------------------------------------------------

        print("1.1. endpoint: company.indexSearch (if enabled in __init__.searchType)")
        try:
            response_index_search = self.index_search(
                country_code="AT",
                search_method="name",
                search_value="360kompany")
        except Exception as e:
            print("step 1.1. - ERROR when trying to hit indexSearch")
            print(e)
            quit()

        print("1.2. endpoint: company.liveSearch (if enabled in __init__.searchType)")
        try:
            response_live_search = self.live_search(
                country_code="AT",
                search_method="name",
                search_value="360kompany")
        except Exception as e:
            print("step 1.2. - ERROR when trying to hit liveSearch")
            print(e)
            quit()

        print("1.3. saving kompanyId to a variable (response_kompany_id)")
        try:
            if self.searchType == 'indexSearch':
                response_kompany_id = response_index_search["searchResults"][0]["kompanyId"]
            elif self.searchType == 'liveSearch':
                response_kompany_id = response_live_search["searchResults"][0]["kompanyId"]
            else:
                print("step 1.3. - ERROR when trying to identify which search type has been executed")
                quit()
            print("> response_kompany_id (kompanyId): ", response_kompany_id)
        except Exception as e:
            print("step 1.3. - ERROR when trying to retrieve kompanyId")
            print(e)
            quit()

        # ------------------------------------------------------------------------------
        print("2. verifying the company")
        # ------------------------------------------------------------------------------

        print("2.1. endpoint: company.dataset")
        try:
            response_dataset = self.dataset(kompany_id=response_kompany_id, dataset_name=self.datasetType)

        except Exception as e:
            print("step 2. - ERROR when trying to hit dataset")
            print(e)
            quit()

        # ------------------------------------------------------------------------------
        print("3. preparation for ordering a certain pdf document for the company")
        # ------------------------------------------------------------------------------

        print("3.1. endpoint: system.catalog")
        try:
            response_catalog = self.catalog(country_code="AT")

        except Exception as e:
            print("step 3.1. - ERROR when trying to hit catalog")
            print(e)
            quit()

        print("3.2. saving productType to a variable (response_product_type)")
        try:
            # the below part seeks for the productType of Register Report
            for product_object in response_catalog:
                if product_object["productName"] == "Register Report":
                    response_product_type = product_object["productType"]

            print("> response_product_type (productType): ", response_product_type)
        except Exception as e:
            print("step 3.2. - ERROR when trying to retrieve productType")
            print(e)
            quit()

        print('3.3. endpoint: product.search')
        try:
            response_product_search = \
                self.product_search(kompany_id=response_kompany_id, product_type=response_product_type)

        except Exception as e:
            print("step 3.3. - ERROR when trying to hit product_search")
            print(e)
            quit()

        print('3.4. saving availability to a variable & evaluating (response_available)')
        try:
            response_available = response_product_search[0]['availability']

            if response_available:
                print(">" + response_product_type + " is available for the company.")
            else:
                print(">" + + response_product_type +
                      " is NOT available for the company. Product order makes no sense, quiting.")
                quit()

        except Exception as e:
            print("step 3.4. - ERROR when trying to retrieve availability")
            print(e)
            quit()

        # ------------------------------------------------------------------------------
        print("4. ordering a certain pdf document for the company")
        # ------------------------------------------------------------------------------

        print('4.1. endpoint: product.order (POST)')
        try:
            response_product = self.product_order(kompany_id=response_kompany_id, product_type=response_product_type)

        except Exception as e:
            print("step 4.1. - ERROR when trying to hit product_order POST")
            print(e)
            quit()

        print("4.2. saving orderId & status to a variable (response_order_id, response_order_status)")
        try:
            # the below part seeks for the orderId & status
            response_order_id = response_product["orderId"]
            response_order_status = response_product["status"]
            print("> response_order_id (orderId): ", response_order_id)
            print("> response_order_status (status): ", response_order_status)
        except Exception as e:
            print("step 4.2. - ERROR when trying to retrieve orderId & status")
            print(e)
            quit()

        print("4.3. endpoint: product.order/{orderId} (GET)")
        try_count = 0
        while response_order_status != 'COMPLETED' and try_count <= self.max_try_count:
            try_count += 1
            print("> checking order status, #", str(try_count))

            try:
                response_product_bookshelf = \
                    self.product_bookshelf(order_id=response_order_id, try_counter=str(try_count))
                try:
                    response_order_status = response_product_bookshelf["status"]
                except Exception as e:
                    response_order_status = "NOT_OK"
                    print("> checking order status FAILED, #", str(try_count))
                    print(e)
                    # todo, here is a known bug in endpoint behaviour, will be resolved soon

                time.sleep(self.try_delay_seconds)
            except Exception as e:
                print("step 4.3. - ERROR when trying to hit product_order GET")
                print(e)
                quit()

        print("4.4. endpoint: product.order/{orderId}/content (GET)")
        try:
            response_product_content = self.product_content_retrieval(order_id=response_order_id)
            if response_product_content:
                print("Saving the pdf document to disk was successful.")
            else:
                print("ERROR - Saving the pdf document to disk was NOT successful.")
        except Exception as e:
            print("step 4.4. - ERROR when trying to retrieve product order content (pdf)")
            print(e)
            quit()


if __name__ == "__main__":
    x = KycApiHelloWorld()
    x.business_logic()
