# Overview
The KycApiHelloWorld script is aimed to introduce a typical business flow and usage of the KYC API from 360kompany,a Moody's Analytics Company.
The script contains a simplified flow with the goal to give a better understanding of the logic of the KYC-API.
For further information please visit the Guides section on https://developer.kompany.com. 

# How to use this script
- Sign up on https://developer.kompany.com
- Navigate to the Dashboard and add (at least a) sandbox plan to your board
- Your api-key is going to be visible on your dashboard
- In the script; add your api_key to the application_secret variable in the `__init__` function
- Review the variables in the `__init__` function and align the settings by just changing True/False or simple string values
- Run the script

# Disclaimer
- The script serves representative purposes for KYC API, therefore it does not strictly follow programming standards.
- The script implements a simplified use case, and it is not supposed to be used in a production environment.
- The particular implementation should follow the exact customer needs, further endpoint calls, error handling, etc. may be necessary on top of what this snippet introduces. 

# Version
- v1.0,- initial public version in Python, published on 20.04.2023
- v1.1 - fixed a bug on product search endpoint call, aligned error handling
- v1.2 - fixes on dataset & productOrder request, minor update on readMe


# The implemented business flow
## 1. identifying the company
KYC API offers indexSearch and liveSearch endpoints for identifying an entity (a company).

    Executing both indexSearch and liveSearch is typically unnecessary, however technically possible.
    Therefore, in the `__init__` method the searchType should be set.
    From technical perspective  (endpoint parameters, response structure) indexSearch and liveSearch behave alike.
    (There are only minor differences such as countryCode is not mandatory for indexSearch, but it's mandatory for liveSearch.)

### 1.1. endpoint: company.indexSearch 
    The endpoint retrieves results from KYC API's company index.
    Parameters:
        - countryCode (optional): 2 or 5-digit ISO codes, like "AT" or "US-DE"
        - searchMethod: enumeration: name|number
        - searchValue: the search string i.e. "360kompany" or "375714x"
    Further notes:
        - the freshness of the data of indexSearch may or may not be sufficient for the actual use case
        - in the response "lastCheckedTime" describes when was the particular company data last time checked directly from the source.
        - indexSearch is typically cheaper and gives a quicker response time compared to liveSearch

### 1.2. endpoint: company.liveSearch
    The endpoint retrieves results directly from the source (except when using a sandbox key).
    Parameters:
        - countryCode (mandatory): 2 or 5-digit ISO codes, like "AT" or "US-DE"
        - searchMethod: enumeration: name|number
        - searchValue: the search string i.e. "360kompany" or "375714x"

### 1.3. saving kompanyId to a variable
    In this step kompanyId, the unique id of the company entity is saved into variable; response_kompany_id.

##  2. verifying the company
### 2.1. endpoint: company.dataset (POST)
    The dataset endpoint provides detailed, structured information about the company in question.
    Parameters:
        - kompanyId: the unique id of the entity in question, the output value from previous step should be used
        - datasetName: enumeration: refresh|full (refresh dataset is retrieved directly from the primary source, full dataset is retrieved from our company index)

    The particular jurisdiction gives a syncronous - immediate response. (In other words: status is immediately 'COMPLETED' and the content datapoint contains the company dataset information)
    If the jurisdiction would be an asyncronous one, at this point a polling would be necessary against the dataset endpoint (GET); with datasetId as parameter.
    The method is similar as in the case of productOrder in 4.3.

## 3. preparation for ordering a certain pdf document for the company 
The aim is a purchase & retrieval of a (most likely pdf) product for a certain company.
For that, the certain company is already known, in this step the productType will be identified and checked in terms of availability.

### 3.1. endpoint: system.catalog
    To get information about the (theoretically) available productTypes for the specific country the catalog endpoint should be called
    Please note: as productTypes typically don't change, this part possibly can be hardcoded

### 3.2. saving productType to a variable
    For a productOrder typically two parameters are needed: kompanyId and productType.
    As the company has been identified (response_kompany_id has been set), now the second parameter need to be set.
    Now productType, (aka. SKU) the unique id of the (pdf) product will be saved to a variable; response_product_type.

### 3.3. endpoint: product.search
    For making sure if the certain (pdf) product is available for the certain company, the productSearch endpoint should be used
    Parameters:
        - kompanyId: the unique id of the entity in question
        - productType: the unique id of the (pdf) document (product); the output value from previous step should be used

### 3.4. saving availability to a variable & evaluating
    From the product search response the datapoint called 'availability' need to be evaluated.
    Step forward to the product order itself obviously only makes sense when the value here is 'true'

## 4. ordering a certain pdf document for the company
The following steps will be executed to retrieve a (pdf) document
- ordering a product
- checking the status of the product order
- retrieving / saving the pdf document

### 4.1. endpoint: product.order (POST)
    With this step a productOrder purchase is executed.
    The response contains the status of the order process (most probably PROCESSING), and (among other infos) the orderId which can be used for the later status checks and also for retrieval. 
    Parameters:
        - kompanyId: the unique id of the entity in question
        - productType: the unique id of the (pdf) document (product); the output value from previous step should be used


### 4.2. saving orderId & status to a variable 
    In this step orderId, is saved into variable; response_order_id.

### 4.3. endpoint: product.order/{orderId} (GET)
    If the status of the order in the previous call was not immediately COMPLETED,
    then current status of the certain order is going to be checked in a while loop.
    (Maximal limit of retrial and volume of the delay between the loops can be set in 
    the __init__ function, by manipulating the `max_try_count` and `try_delay_seconds` variables.)
    
    This endpoint can be called without the orderId parameter in the path.
    In that case the previously purchased orders are listed (sorted by request-time desc), with the possibility to filter the
    previously purchased orders by different attributes, such as status or productType.

### 4.4. endpoint: product.order/{orderId}/content (GET)
    Once the status of the order turned to COMPLETED, the pdf document can be retrieved by this endpoint.
    For the call not only the orderId but mimeType is also required.
    The available mimeType can be checked in the product order response (both with POST, GET/{orderId} and GET methods)
    
    As in case of a typical product order, the content is most probably pdf, the hit gives positive result either 
    with `application/pdf` as mimeType or if mimeType is not set.

    If the request was successful the pdf document will be saved to the disk.